#include "gfx.h"
#include <stdio.h>
#include "mz80.h"
#include "yaame.h"

static unsigned char amidarcolour[] =
{
	/* palette */
	0x00,0x07,0xC0,0xB6,
	0x00,0x38,0xC5,0x67,
	0x00,0x30,0x07,0x3F,
	0x00,0x07,0x30,0x3F,
	0x00,0x3F,0x30,0x07,
	0x00,0x38,0x67,0x3F,
	0x00,0xFF,0x07,0xDF,
	0x00,0xF8,0x07,0xFF
};

UINT8 amidarreadin0(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_SPACE])
           temp&=~8;
        if (keys[KEY_RIGHT])
           temp&=~0x10;
        if (keys[KEY_LEFT])
           temp&=~0x20;
        if (keys[KEY_C])
           temp&=~0x80;
        return temp;
}

UINT8 amidarreadin1(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xFF;
        if (keys[KEY_1])
           temp&=~0x80;
        return temp;
}

UINT8 amidarreadin2(UINT32 addr, struct MemoryReadByte *m)
{
        unsigned char temp=0xF1;
        if (keys[KEY_UP])
           temp&=~0x10;
        if (keys[KEY_DOWN])
           temp&=~0x40;
        return temp;
}

UINT8 amidarreaddsw(UINT32 addr, struct MemoryReadByte *m)
{
        return 0xFF;
}

struct MemoryReadByte amidarread[] =
{
        {0xB000,0xB000,amidarreadin0},
        {0xB010,0xB010,amidarreadin1},
        {0xB020,0xB020,amidarreadin2},
        {0xB820,0xB820,amidarreaddsw},
        {-1,-1,NULL}
};

struct MemoryWriteByte amidarwrite[] =
{
        {0,0x4000,writerom},
        {0xA008,0xA008,nmienable},
        {-1,-1,NULL}
};

struct z80PortRead amidarportread[] =
{
	{-1,	  -1,	  NULL}
};

struct z80PortWrite amidarportwrite[] =
{
	{-1,	-1,	NULL}
};

int amidarloadgfx()
{
        FILE *f,*ff;
        int y,c;
        buffer=createbmp(512,512);
        clearall(buffer,0);
        f=fopen("amidar.5f","rb");
        ff=fopen("amidar.5h","rb");
        for (c=0;c<256;c++)
        {
                for (y=7;y>-1;y--)
                {
                        bitplane[0][c][y]=getc(f);
                        bitplane[1][c][y]=getc(ff);
                }
        }
        fclose(ff);
        fclose(f);
	for (c=0;c<32;c++)
	{
		int bit0,bit1,bit2;
		bit0 = (amidarcolour[c] >> 0) & 0x01;
		bit1 = (amidarcolour[c] >> 1) & 0x01;
		bit2 = (amidarcolour[c] >> 2) & 0x01;
		yaamepal[c].r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		bit0 = (amidarcolour[c] >> 3) & 0x01;
		bit1 = (amidarcolour[c] >> 4) & 0x01;
		bit2 = (amidarcolour[c] >> 5) & 0x01;
		yaamepal[c].g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
		bit0 = 0;
		bit1 = (amidarcolour[c] >> 6) & 0x01;
		bit2 = (amidarcolour[c] >> 7) & 0x01;
		yaamepal[c].b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
                yaamepal[c].r/=4;
                yaamepal[c].g/=4;
                yaamepal[c].b/=4;
	}
        return 0;
}

int amidarinit()
{
        printf("Loading amidar ROMs\n");
        loadrom("amidar.2c",0,4096);
        loadrom("amidar.2e",0x1000,4096);
        loadrom("amidar.2f",0x2000,4096);
        loadrom("amidar.2h",0x3000,4096);
        amidarloadgfx();
        initz80(amidarread,amidarwrite,amidarportread,amidarportwrite);
        printf("amidar loaded\n");
        nmi=0;
        return 0;
}

void amidardraw()
{
        int x,y,colset,c,tile,flipx;
        int pos=0x9000,pos2=0x9800;
        for (x=0;x<32;x++)
        {
                colset=ram[pos2++];
                pos2++;
                for (y=0;y<32;y++)
                {
                        drawtile(ram[pos++],(31-x)*8,y*8,colset);
                }
        }
        for (c=0x9840;c<0x985F;c+=4)
        {
                y=ram[c+3]+1;
                x=ram[c];
                if (x>1&&y>1)
                {
                        tile=ram[c+1]&0x3F;
                        tile<<=2;
                        colset=ram[c+2]&7;
                        flipx=ram[c+1]&0x40;
                        if (!flipx)
                        {
                                drawsprite(tile,x+8,y,colset,0,0);
                                drawsprite(tile+2,x,y,colset,0,0);
                                drawsprite(tile+1,x+8,y+8,colset,0,0);
                                drawsprite(tile+3,x,y+8,colset,0,0);
                        }
                        else
                        {
                                drawsprite(tile,x,y,colset,1,0);
                                drawsprite(tile+2,x+8,y,colset,1,0);
                                drawsprite(tile+1,x,y+8,colset,1,0);
                                drawsprite(tile+3,x+8,y+8,colset,1,0);
                        }
                }
        }
        waitforsync();
        blit(buffer,screen,0,0,0,0,256,256);
}

void amidarrun()
{
        while (!keys[KEY_ESC])
        {
                if (mz80exec(51200)!=0x80000000)
                {
                        printf("It crashed\n");
                        exit(-1);
                }
                amidardraw();
                if (nmi)
                   mz80nmi();
        }
}
