#include "gfx.h"
#include <stdio.h>
#include "mz80.h"
#include "yaame.h"

volatile int yaamedraw;
int temp;
volatile void drawupdate()
{
        yaamedraw=1;
}

END_OF_FUNCTION(drawupdate);

void writerom(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
}

void nmienable(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        nmi=data;
}

void loadrom(char *fn, int addr, int size)
{
        FILE *f=fopen(fn,"rb");
        if (!f)
        {
                printf("File %s not found\n",fn);
                chdir("..");
                exit(-1);
        }
        fread(ram+addr,size,1,f);
        fclose(f);
}

/*Draws a tile onto the back buffer from the bitplane structure*/
void drawtile(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if ((xx+y)>0&&(x+yy)>0)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

/*Draws a tile onto the back buffer from the bitplane structure with masking*/
void drawtilemask(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

/*Draw a Galaxian style sprite onto the back buffer with masking*/
void drawsprite(int tile, unsigned char x, unsigned char y, unsigned char colset, int xflip, int yflip)
{
        int xx,yy;
        unsigned char byte1,byte2;
        if (!xflip)
        {
                for (yy=0;yy<8;yy++)
                {
                        byte2=bitplane[0][tile][yy];
                        byte1=bitplane[1][tile][yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
        else
        {
                for (yy=7;yy>-1;yy--)
                {
                        byte2=bitplane[0][tile][7-yy];
                        byte1=bitplane[1][tile][7-yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
}

void drawtile2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

void drawtilemask2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

void initz80(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite)
{
        memset(&z80,0,sizeof(z80));
        z80.z80Base=ram;
        z80.z80IoRead=portread;
        z80.z80IoWrite=portwrite;
        z80.z80MemRead=read;
        z80.z80MemWrite=write;
	mz80SetContext(&z80);
	mz80reset();
	mz80GetContext(&z80);
	z80.z80intAddr=0x38;
	z80.z80nmiAddr=0x66;
	mz80SetContext(&z80);
}

void init8080(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite)
{
        memset(&z80,0,sizeof(z80));
        z80.z80Base=ram;
        z80.z80IoRead=portread;
        z80.z80IoWrite=portwrite;
        z80.z80MemRead=read;
        z80.z80MemWrite=write;
	mz80SetContext(&z80);
	mz80reset();
	mz80GetContext(&z80);
	z80.z80intAddr=0x8;
	z80.z80nmiAddr=0x10;
	mz80SetContext(&z80);
}

void inityaame()
{
        yaamedraw=0;
        LOCK_VARIABLE(yaamedraw);
        LOCK_FUNCTION(drawupdate);
        installint(drawupdate,BPS_TO_TIMER(60));
}

void waitforsync()
{
        while (!yaamedraw);
        yaamedraw=0;
}
