#include <pc.h>
#include <dos.h>

void fmwrite(unsigned char reg, unsigned char val)
{
        int c;
        for (c=0;c<35;c++)
            inportb(0x388);
        outportb(0x388,reg);
        for (c=0;c<7;c++)
            inportb(0x388);
        outportb(0x389,val);
}

struct
{
  float stopFreq; 
  float freqMult;
}  blockData[8] = {
    { 48.50336838, 21.09131869 },
    { 97.00673676, 10.54565935 },
    { 194.0134735, 5.272829673 },
    { 388.026947,  2.636414836 },
    { 776.053894,  1.318207418 },
    { 1552.107788, 0.659103709 },
    { 3104.215576, 0.329551854 },
    { 6208.431152, 0.164775927 }
};

void fmtone(int voice, int fnum, int octave)
{
        unsigned char temp=((fnum>>8)&3)|((octave&7)<<2)|32;
        fmwrite(0x20+voice,0x01);
        fmwrite(0x40+voice,0x10);
        fmwrite(0x60+voice,0xF0);
        fmwrite(0x80+voice,0x77);
        fmwrite(0xA0+voice,fnum&0xFF);
        fmwrite(0x23+voice,0x01);
        fmwrite(0x43+voice,0x00);
        fmwrite(0x63+voice,0xF0);
        fmwrite(0x83+voice,0x77);
        fmwrite(0xB0+voice,temp);
}

unsigned char fmread(unsigned char reg)
{
        int c;
        for (c=0;c<35;c++)
            inportb(0x388);
        outportb(0x388,reg);
        for (c=0;c<7;c++)
            inportb(0x388);
        return inportb(0x389);
}

int fmdetect()
{
   fmwrite(1, 0);                        /* init test register */

   fmwrite(4, 0x60);                     /* reset both timers */
   fmwrite(4, 0x80);                     /* enable interrupts */

   if (inportb(0x388) & 0xE0)
      return 0;

   fmwrite(2, 0xFF);                     /* write 0xFF to timer 1 */
   fmwrite(4, 0x21);                     /* start timer 1 */

   delay(1);

   if ((inportb(0x388) & 0xE0) != 0xC0)
      return 0;

   fmwrite(4, 0x60);                     /* reset both timers */
   fmwrite(4, 0x80);                     /* enable interrupts */

   return 1;
}

void stopnote(int voice)
{
        fmwrite(0xB0+voice,0);
}

void allnotesoff()
{
        int c;
        for (c=0;c<8;c++)
            stopnote(c);
}

void playnote(unsigned char voice, float freq)
{
   int blockNum, fNum;
   if (!freq)
   {
        stopnote(voice);
        return;
   }
   for (blockNum=0; blockNum<8; blockNum++) {
      if (freq < blockData[blockNum].stopFreq) {
         fNum = (int)(freq*blockData[blockNum].freqMult);
         fmtone(voice, fNum, blockNum);
         return;
      }
   }
   closegfx();
   printf("%ihz out of range\n",freq);
   exit(-1);
}

void fmreset()
{
        int c;
        for (c=0;c<0xFF;c++)
            fmwrite(c,0);
}

int initfm()
{
        if (!fmdetect())
           return -1;
        fmreset();
        return 0;
}
