#include <tomgfx.h>
#include <stdio.h>
#include "mem.h"
#include "mz80.h"
#include "yaame.h"

FILE *tilelog;
BMP *sprite;

int scrollx;
unsigned char dosprite[128][16][4];
int mrdospritelookup[64];

inline int mrdodecode(int val)
{
        int temp=(val&16)?2:0;
        return temp|(val&1);
}

static void writescrollx(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite)
{
        scrollx=data;
}

UINT8 readmrdo(UINT32 addr, struct MemoryReadByte *crap)
{
        mz80GetContext(&z80);
        return ram[z80.z80hl.hl];
}

UINT8 readmrdoin0(UINT32 addr, struct MemoryReadByte *crap)
{
        unsigned char temp=0xFF;
        if (keys[KEY_1])
           temp&=~32;
        if (keys[KEY_UP])
           temp&=~8;
        if (keys[KEY_DOWN])
           temp&=~2;
        if (keys[KEY_LEFT])
           temp&=~1;
        if (keys[KEY_RIGHT])
           temp&=~4;
        if (keys[KEY_SPACE])
           temp&=~16;
        return temp;
}

UINT8 readmrdoin1(UINT32 addr, struct MemoryReadByte *crap)
{
        unsigned char temp=0xFF;
        if (keys[KEY_C])
           temp&=~64;
        return temp;
}

struct MemoryWriteByte mrdowrite[] =
{
        {0,0x8000,writerom},
        {0xF800,0xF800,writescrollx},
        {-1,-1,NULL}
};

struct MemoryReadByte mrdoread[] =
{
        {0xA000,0xA000,readmrdoin0},
        {0xA001,0xA001,readmrdoin1},
        {0xA002,0xA002,readmrdoin1},
        {0xA003,0xA003,readmrdoin1},
        {0x9800,0xB000,readmrdo},
        {-1,-1,NULL}
};

UINT16 readmrdoport(UINT16 port, struct z80PortRead *pPR)
{
        closegfx();
        printf("Reading port %04X\n",port);
        exit(0);
}

struct z80PortRead mrdoportread[] =
{
        {0,0xFFFF,readmrdoport},
	{-1,	  -1,	  NULL}
};

static void writemrdoport(UINT16 port, UINT8 data, struct z80PortWrite *pPW)
{
        closegfx();
        printf("Writing %02X to port %04X\n",data,port);
        exit(0);
}

struct z80PortWrite mrdoportwrite[] =
{
        {0,0xFFFF,writemrdoport},
	{-1,	-1,	NULL}
};

int loadmrdogfx()
{
        FILE *f,*ff;
        int x,y,c,xx;
        unsigned char byte1,byte2;
        buffer=createbmp(512,512);
        f=fopen("s8-09.bin","rb");
        ff=fopen("u8-10.bin","rb");
        for (c=0;c<256;c++)
        {
                for (y=0;y<8;y++)
                {
                        bitplane[0][c][y]=getc(f);
                        bitplane[1][c][y]=getc(ff);
                }
        }
        fclose(ff);
        fclose(f);
        f=fopen("r8-08.bin","rb");
        ff=fopen("n8-07.bin","rb");
        for (c=0;c<256;c++)
        {
                for (y=0;y<8;y++)
                {
                        bitplane[0][c+256][y]=getc(f);
                        bitplane[1][c+256][y]=getc(ff);
                }
        }
        fclose(ff);
        fclose(f);
        f=fopen("h5-05.bin","rb");
        for (c=0;c<64;c++)
        {
                for (y=0;y<16;y++)
                {
                        for (x=0;x<16;x+=4)
                        {
                                dosprite[c][y][x>>2]=getc(f);
                        }
                }
        }
        fclose(f);
        f=fopen("k5-06.bin","rb");
        for (c=0;c<64;c++)
        {
                for (y=0;y<16;y++)
                {
                        for (x=0;x<16;x+=4)
                        {
                                dosprite[c+64][y][x>>2]=getc(f);
                        }
                }
        }
        fclose(f);
        return 0;
}

static unsigned char mrdocolour[] =
{
	/* palette (high bits) */
	0x00,0x0C,0x03,0x00,
	0x0F,0x0B,0x0C,0x3F,
	0x0D,0x0F,0x0F,0x0C,
	0x0C,0x3C,0x0C,0x30,
	0x0C,0x03,0x30,0x03,
	0x0C,0x0F,0x00,0x3F,
	0x03,0x1E,0x00,0x0F,
	0x37,0x36,0x0D,0x33,
	/* palette (low bits) */
	0x00,0x0C,0x03,0x00,
	0x0C,0x03,0x00,0x3F,
	0x0F,0x03,0x0F,0x3F,
	0x0C,0x0F,0x0F,0x3A,
	0x03,0x0F,0x00,0x0C,
	0x00,0x0F,0x3F,0x03,
	0x2A,0x0C,0x00,0x0A,
	0x0C,0x0E,0x3F,0x0F,
	/* sprite color lookup table */
	0x00,0x97,0x71,0xF9,
	0x00,0x27,0xA5,0x13,
	0x00,0x32,0x77,0x3F,
	0x00,0xA7,0x72,0xF9,
	0x00,0x1F,0x9A,0x77,
	0x00,0x15,0x27,0x38,
	0x00,0xC2,0x55,0x69,
	0x00,0x7F,0x76,0x7A
};

void loadpal()
{
        int c,i,j;
        for (i = 0;i < 64;i++)
	{
		for (j = 0;j < 4;j++)
		{
			int bit0,bit1,bit2,bit3;
			bit0=(mrdocolour[4*(i/8)+j+32]>>1)&0x01;
			bit1=(mrdocolour[4*(i/8)+j+32])&0x01;
			bit2=(mrdocolour[4*(i%8)+j]>>1)&0x01;
			bit3=(mrdocolour[4*(i%8)+j])&0x01;
			yaamepal[(4*i + j)].r = 0x2c * bit0 + 0x37 * bit1 + 0x43 * bit2 + 0x59 * bit3;
			bit0=(mrdocolour[4*(i / 8) + j + 32] >> 3) & 0x01;
			bit1=(mrdocolour[4*(i / 8) + j + 32] >> 2) & 0x01;
			bit2=(mrdocolour[4*(i % 8) + j] >> 3) & 0x01;
			bit3=(mrdocolour[4*(i % 8) + j] >> 2) & 0x01;
			yaamepal[(4*i + j)].g = 0x2c * bit0 + 0x37 * bit1 + 0x43 * bit2 + 0x59 * bit3;
			bit0=(mrdocolour[4*(i / 8) + j + 32] >> 5) & 0x01;
			bit1=(mrdocolour[4*(i / 8) + j + 32] >> 4) & 0x01;
			bit2=(mrdocolour[4*(i % 8) + j] >> 5) & 0x01;
			bit3=(mrdocolour[4*(i % 8) + j] >> 4) & 0x01;
			yaamepal[(4*i + j)].b = 0x2c * bit0 + 0x37 * bit1 + 0x43 * bit2 + 0x59 * bit3;
		}
	}
        for (c=0;c<256;c++)
        {
                yaamepal[c].r>>=2;
                yaamepal[c].g>>=2;
                yaamepal[c].b>>=2;
        }
        for (c=0;c<32;c++)
        {
                int bits1,bits2;
                bits1 = (mrdocolour[c + 64] >> 0) & 3;
		bits2 = (mrdocolour[c + 64] >> 2) & 3;
                mrdospritelookup[c]=bits1 + (bits2 << 2) + (bits2 << 5);
                bits1 = (mrdocolour[c + 64] >> 4) & 3;
		bits2 = (mrdocolour[c + 64] >> 6) & 3;
                mrdospritelookup[c+32]=bits1 + (bits2 << 2) + (bits2 << 5);
        }
}

int mrdoinit()
{
        FILE *f;
        printf("Loading Mr Do! ROMs\n");
        f=fopen("a4-01.bin","rb");
        fread(ram,8192,1,f);
        fclose(f);
        f=fopen("c4-02.bin","rb");
        fread(ram+0x2000,8192,1,f);
        fclose(f);
        f=fopen("e4-03.bin","rb");
        fread(ram+0x4000,8192,1,f);
        fclose(f);
        f=fopen("f4-04.bin","rb");
        fread(ram+0x6000,8192,1,f);
        fclose(f);
        loadmrdogfx();
        loadpal();
        memset(&z80,0,sizeof(z80));
        z80.z80Base=ram;
        z80.z80IoRead=mrdoportread;
        z80.z80IoWrite=mrdoportwrite;
        z80.z80MemRead=mrdoread;
        z80.z80MemWrite=mrdowrite;
	mz80SetContext(&z80);
	mz80reset();
	mz80GetContext(&z80);
	z80.z80intAddr=0x38;
	z80.z80nmiAddr=0x66;
	mz80SetContext(&z80);
        printf("Mr Do! loaded\n");
        return 0;
}

inline void drawmrdosprite(int c, int x, int y, unsigned char colset, int xflip, int yflip)
{
        unsigned char temp,temp2;
        int xx,yy,xxx;
        for (yy=0;yy<16;yy++)
        {
                for (xx=0;xx<16;xx+=4)
                {
                        if (xflip)
                           temp=dosprite[c][15-yy][(15-xx)>>2];
                        else
                           temp=dosprite[c][yy][(15-xx)>>2];
                        for (xxx=0;xxx<4;xxx++)
                        {
                                temp2=mrdodecode(temp);
                                if (temp2)
                                {
                                        if (yflip)
                                           buffer->line[(15-xx)+(xxx)+y][yy+x]=(temp2)|(colset<<2)+64;
                                        else
                                           buffer->line[xx+(3-xxx)+y][yy+x]=(temp2)|(colset<<2)+64;
                                }
                                temp>>=1;
                        }
                }
        }
}

void mrdodraw()
{
        int y,x;
        int pos,pos2,pos3,pos4;
        unsigned char c;
        int xflip,yflip;
        char s[80];
        pos=0x8C00;
        pos2=0x8800;
        pos3=0x8400;
        pos4=0x8000;
        for (y = 0; y <32; y++)
        {
                for (x = 31; x >-1; x--)
                {
                      c = ram[pos3];
                      sprintf(s,"%02X %i %i\n",c+(ram[pos4]&0x80)?256:0,(y<<3)-scrollx,x<<3);
                      fputs(s,tilelog);
                      if (ram[pos4]&0x80)
                         drawtile(c,(y<<3)-scrollx,x<<3,ram[pos4]&0x7F);
                      else
                         drawtile(c+256,(y<<3)-scrollx,x<<3,ram[pos4]&0x7F);
                      pos3++;
                      pos4++;
                }
        }
        for (y = 0; y <32; y++)
        {
                for (x = 31; x >-1; x--)
                {
                      c = ram[pos];
                      sprintf(s,"%02X %i %i\n",c+(ram[pos2]&0x80)?256:0,(y<<3)-scrollx,x<<3);
                      fputs(s,tilelog);
                      if (ram[pos2]&0x80)
                         drawtilemask(c+256,y<<3,x<<3,ram[pos2]&0x7F);
                      else
                         drawtilemask(c,y<<3,x<<3,ram[pos2]&0x7F);
                      pos++;
                      pos2++;
                }
        }
        for (c=0;c<64;c++)
        {
                if (ram[(c<<2)+0x9000]!=0)
                {
                        x=256-ram[(c<<2)+0x9001];
                        y=240-ram[(c<<2)+0x9003];
                        xflip=ram[(c<<2)+0x9002]&0x20;
                        yflip=ram[(c<<2)+0x9002]&0x10;
                        if (x>0 && y>0 && x<256 && y<256)
                           drawmrdosprite(ram[(c<<2)+0x9000]&127,x,y,/*ram[(c<<2)+0x9002]&0xF*/mrdospritelookup[ram[(c<<2)+0x9002]&0xF],xflip,yflip);
                }
        }
        blit(buffer,screen,32,0,0,0,192,256);
}

void mrdorun()
{
        FILE *f;
        int c;
        tilelog=fopen("mrdo.log","wb");
        for (c=0;c<256;c++)
            putc(0,tilelog);
        if (!tilelog)
           exit(-1);
        putc('X',tilelog);
        while (!keys[KEY_ESC])
        {
                if (mz80exec(57000)!=0x80000000)
                {
                        printf("It crashed\n");
                        exit(-1);
                }
                mz80int(1);
                mrdodraw();
        }
}
