/*The z80 context*/
struct mz80context z80;

/*Graphics structure*/
unsigned char bitplane[2][768][8];
/*The emulated memory*/
unsigned char ram[65536];

unsigned char nmi;
/*Graphics palette and back buffer*/
PALETTE yaamepal;
BMP *buffer;

void inityaame();
void waitforsync();

void initz80(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite);
void init8080(struct MemoryReadByte *read, struct MemoryWriteByte *write, struct z80PortRead *portread, struct z80PortWrite *portwrite);

/*Common memory write functions*/
void writerom(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite);
void nmienable(UINT32 addr, UINT8 data, struct MemoryWriteByte *pMemWrite);

/*Draws a tile onto the back buffer from the bitplane structure*/
extern inline void drawtile(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

/*Draws a tile onto the back buffer from the bitplane structure with masking*/
extern inline void drawtilemask(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=7;xx>-1;xx--)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

/*Draw a Galaxian style sprite onto the back buffer with masking*/
extern inline void drawsprite(int tile, unsigned char x, unsigned char y, unsigned char colset, int xflip, int yflip)
{
        int xx,yy;
        unsigned char byte1,byte2;
        if (!xflip)
        {
                for (yy=0;yy<8;yy++)
                {
                        byte2=bitplane[0][tile][yy];
                        byte1=bitplane[1][tile][yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
        else
        {
                for (yy=7;yy>-1;yy--)
                {
                        byte2=bitplane[0][tile][7-yy];
                        byte1=bitplane[1][tile][7-yy];
                        if (!yflip)
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                        else
                        {
                                for (xx=7;xx>-1;xx--)
                                {
                                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                                           buffer->line[(7-xx)+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                                        byte1>>=1;
                                        byte2>>=1;
                                }
                        }
                }
        }
}

extern inline void drawtile2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}

extern inline void drawtilemask2(int tile, unsigned char x, unsigned char y, unsigned char colset)
{
        int xx,yy;
        unsigned char byte1,byte2;
        for (yy=0;yy<8;yy++)
        {
                byte2=bitplane[0][tile][yy];
                byte1=bitplane[1][tile][yy];
                for (xx=0;xx<8;xx++)
                {
                        if (((byte1&1)|((byte2&1)<<1)|(colset<<2))&3)
                           buffer->line[xx+y][x+yy]=(byte1&1)|((byte2&1)<<1)|(colset<<2);
                        byte1>>=1;
                        byte2>>=1;
                }
        }
}
